use crate::{
    common::{Section, SectionIterResult},
    error::Error,
};
use xmas_elf::sections::SectionIter as ElfIter;
use xmas_elf::ElfFile;

pub struct SectionIter<'data> {
    pub elf: &'data ElfFile<'data>,
    pub iter: ElfIter<'data, 'data>,
}

impl<'data> Iterator for SectionIter<'data> {
    type Item = Result<Section<'data>, Error>;

    fn next(&mut self) -> Option<Self::Item> {
        match Section::iterate(&self.elf, &mut self.iter) {
            SectionIterResult::Err(err) => Some(Err(err)),
            SectionIterResult::None => None,
            SectionIterResult::Skip => self.next(),
            SectionIterResult::Ok(r) => Some(Ok(r)),
        }
    }
}
